#define _CRT_SECURE_NO_WARNINGS
//
// Copyright (c) 2004 - InfoMus Lab - DIST - University of Genova
//
// InfoMus Lab (Laboratorio di Informatica Musicale)
// DIST - University of Genova
//
// http://www.infomus.dist.unige.it
// news://infomus.dist.unige.it
// mailto:staff@infomus.dist.unige.it
//
// Developer: Gualtiero Volpe
// mailto:volpe@infomus.dist.unige.it
//
// Developer: Trey Harrison
// mailto:trey@treyharrison.com
//
// Last modified: Oct. 26 2006
//

#include "PluginBase.h"
#include <stdio.h>
#include <memory.h>
#include <algorithm>
#include "ffgl/ParamBase.h"
#include "ffgl/ParamBool.h"
#include "ffgl/ParamFloat.h"
#include "ffgl/ParamRange.h"
#include "ffgl/ParamText.h"

namespace ffgl
{
// String used by GetParamDisplay() for returning display value to the FFGL host
static std::string displayString;

////////////////////////////////////////////////////////
// PluginBase constructor and destructor
////////////////////////////////////////////////////////

PluginBase::PluginBase() :
	PluginManager(),
	bpm( 120.0f ),
	barPhase( 0.0f )
{
}

PluginBase::~PluginBase()
{
}

////////////////////////////////////////////////////////
// Default implementation of PluginBase methods
////////////////////////////////////////////////////////

const char* PluginBase::GetParamDisplay( FFUInt32 index )
{
	char* retval = (char*)FF_FAIL;

	auto param = GetParam( index );
	if( param )
	{
		const std::string& displayString = param->GetDisplay();
		return displayString.c_str();
	}

	return retval;
}

// 2019-08-22 ffgl@subpixels.com - created
FFResult PluginBase::SetMixedParam( FFUInt32 index, const FFMixed mixed )
{
	ParamBase* param = FindParam( index );
	if( param == nullptr )
		return FF_FAIL;

	return param->SetMixedValue( mixed );
}

// 2019-08-22 ffgl@subpixels.com - created
const FFMixed PluginBase::GetMixedParam( FFUInt32 index ) const
{
	FFMixed retval = { FF_FAIL };

	const ParamBase* param = GetParam( index );
	if( param == nullptr )
		return retval;

	return param->GetMixedValue();
}

InputStatus PluginBase::GetInputStatus( FFUInt32 index )
{
	if( index >= GetMaxInputs() )
		return InputStatus::FF_FAIL;

	return InputStatus::FF_INPUT_INUSE;
}

void PluginBase::SetBeatInfo( float bpm, float barPhase )
{
	this->bpm      = bpm;
	this->barPhase = barPhase;
}

void PluginBase::SetHostInfo( const char* hostname, const char* version )
{
	hostInfos.name    = hostname;
	hostInfos.version = version;
}

void PluginBase::SetSampleRate( FFUInt32 _sampleRate )
{
	sampleRate = _sampleRate;
}

}//End namespace ffgl
